@@ -1,4 +1,4 @@
-//  $Id: command_line.cxx,v 1.6 2003/01/11 16:11:36 grumbel Exp $
+//  $Id: command_line.cxx,v 1.8 2003/07/26 14:43:56 grumbel Exp $
 //
 //  Construo - A wire-frame construction gamee
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -34,6 +34,7 @@ CommandLine::error (const std::string& msg)
 void
 CommandLine::parse (int argc, char** argv)
 {
+  // FIXME: Replace this with getopt_long() or argp()
   for (int i = 1; i < argc; ++i) // Skip Programm name
     {
       //std::cout << "Arg: " << argv[i] << std::endl;
@@ -61,15 +62,24 @@ CommandLine::parse (int argc, char** argv)
             {
               settings.alphablending = false;
             }
-          else if (strcmp(argv[i], "--disable-antialiasing") == 0)
+          else if (strcmp(argv[i], "--disable-antialiasing") == 0
+                   || strcmp(argv[i], "-d") == 0)
             {
               settings.antialiasing = false;
             }
-          else if (strcmp(argv[i], "--disable-doublebuffer") == 0
-              || strcmp(argv[i], "-d") == 0)
+          else if (strcmp(argv[i], "--disable-thicklines") == 0
+                   || strcmp(argv[i], "-l") == 0)
+            {
+              settings.thick_lines = false;
+            }
+          else if (strcmp(argv[i], "--disable-doublebuffer") == 0)
             {
               settings.doublebuffer = false;
             }
+          else if (strcmp(argv[i], "--datadir") == 0)
+            {
+              settings.datadir = argv[++i];
+            }
           else if (strcmp(argv[i], "-g") == 0
                    || strcmp(argv[i], "--geometry") == 0)
             {
@@ -114,10 +124,12 @@ CommandLine::print_help ()
             << "  -h, --help                  display this help text\n"
             << "  -v,--version                Print version number of the programm\n"
             << "  -f, --fullscreen            switch to fullscreen mode if available\n"
-            << "  -d, --disable-doublebuffer  disable the double buffer (might cause flicker)\n"
             << "  -g, --geometry WIDTHxHEIGHT switch resolution to WIDTH and HEIGHT\n"
-            << "  --disable-antialiasing      Switch into non antialiased mode\n"
+            << "  -d, --disable-antialiasing  Switch into non antialiased mode\n"
+            << "  -l, --disable-thicklines    Disable thick (slow) line drawing\n"
+            << "  --disable-doublebuffer  disable the double buffer (might cause flicker)\n"
             << "  --disable-alphablending     Switch into non alphablended mode\n"
+            << "  --datadir DIR               Set the datadir of Construo to DIR\n"
             << std::endl;
 }
 
