@@ -1,4 +1,4 @@
-//  $Id: construo_main.cxx,v 1.22 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: construo_main.cxx,v 1.26 2003/07/26 11:18:47 grumbel Exp $
 //
 //  Construo - A wire-frame construction game
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -22,6 +22,7 @@
 #include "particle.hxx"
 #include "config.h"
 #include "world.hxx"
+#include "path_manager.hxx"
 #include "construo_error.hxx"
 
 #ifdef USE_X11_DISPLAY
@@ -45,8 +46,9 @@ ConstruoMain* construo_main;
 Controller*   controller;
 
 ConstruoMain::ConstruoMain ()
+  : do_quit(false),
+    gui_manager(0)
 {
-  do_quit = false;
 }
 
 ConstruoMain::~ConstruoMain ()
@@ -56,7 +58,16 @@ ConstruoMain::~ConstruoMain ()
 char* 
 ConstruoMain::get_title ()
 {
-  return "Construo";
+  return "Construo " VERSION;
+}
+
+void
+ConstruoMain::exit()
+{
+  on_exit();
+  delete gui_manager;
+  deinit_system();
+  ::exit(EXIT_SUCCESS);
 }
 
 void
@@ -65,9 +76,9 @@ ConstruoMain::on_exit()
   std::cout << "Calling on_exit()" << std::endl;
 
   //if (!controller->has_been_run())
-    {
-      controller->save_world("/user/laststate.construo");
-    }
+  {
+    controller->save_world("/user/laststate.construo");
+  }
 
   std::cout << "\n\n            Thank you for playing Construo!\n\n\n"
             << "  New versions and more information can be found at:\n\n"
@@ -76,32 +87,60 @@ ConstruoMain::on_exit()
             << "    * Ingo Ruhnke <grumbel@gmx.de>\n\n" << std::endl;
 }
 
+void
+ConstruoMain::init_system()
+{
+  std::cout << "ConstruoMain::init_system()" << std::endl;
+  system = new UnixSystem();
+#ifdef USE_X11_DISPLAY
+  display = new X11Display(settings.screen_width, settings.screen_height, 
+                           settings.fullscreen);
+#elif USE_GLUT_DISPLAY
+  display = new GlutDisplay(settings.screen_width, settings.screen_height, settings.fullscreen);
+#else
+#  error "No display type defined"
+#endif
+
+  // Init the display, input systems
+  graphic_context = display;
+  input_context   = display;
+  system_context  = system;
+}
+
+void
+ConstruoMain::deinit_system()
+{
+  std::cout << "ConstruoMain::deinit_system()" << std::endl;
+  delete display;
+  delete system;
+}
+
 int 
 ConstruoMain::main (int argc, char* argv[]) // FIXME: pass an option class, instead command line arguments
 {
   CommandLine::parse(argc, argv);
 
   try {
-#ifdef USE_X11_DISPLAY
-    X11Display display (settings.screen_width, settings.screen_height, 
-                        settings.fullscreen);
-#elif USE_GLUT_DISPLAY
-    GlutDisplay display (settings.screen_width, settings.screen_height);
-#else
-#  error "No display type defined"
-#endif
-    UnixSystem system;
+    // Init the System
+    init_system();
 
-    // Init the display, input systems
-    graphic_context = &display;
-    input_context   = &display;
-    system_context  = &system;
-  
     std::cout << PACKAGE_STRING"\n" << std::endl;
     std::cout << "If you have throuble with programm startup, delete the file:\n\n" 
               << "    " << system_context->get_construo_rc_path() << "laststate.construo\n" << std::endl;
 
-    GUIManager* gui_manager = new GUIManager ();
+    if (!settings.datadir.empty())
+      path_manager.add_path(settings.datadir);
+
+    path_manager.add_path(".");
+    path_manager.add_path("..");
+    path_manager.add_path(CONSTRUO_DATADIR); 
+    if (!path_manager.find_path("examples/"))
+      {
+        std::cout << "Couldn't find Construo Datadir, use '--datadir DIR' to set it manually." << std::endl;
+        ::exit(EXIT_FAILURE);
+      }
+
+    gui_manager = new GUIManager ();
 
     if (!settings.startup_file.empty())
       {
@@ -120,12 +159,11 @@ ConstruoMain::main (int argc, char* argv[]) // FIXME: pass an option class, inst
           }
       }
   
-    display.run();
-
-    on_exit();
-    
-    delete gui_manager;
+    // For some targets this will never return
+    display->run();
 
+    // Shutdown the system and exit
+    exit();
   } catch (ConstruoError& err) {
     std::cout << "Error ocurred: " << err.msg << std::endl;
     return EXIT_FAILURE;
