@@ -1,4 +1,4 @@
-//  $Id: controller.cxx,v 1.9 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: controller.cxx,v 1.11 2003/07/26 14:43:56 grumbel Exp $
 //
 //  Construo - A wire-frame construction game
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -17,6 +17,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#include "worldview_component.hxx"
 #include "construo_error.hxx"
 #include "controller.hxx"
 
@@ -55,6 +56,9 @@ Controller::load_world (const std::string& filename)
 
   std::cout << "Loading World..." << std::endl;
   world = new World (filename);
+
+  WorldViewComponent::instance()->on_world_change();
+
   running = false;
   std::cout << "Loading World... DONE" << std::endl; 
 }
@@ -157,8 +161,10 @@ Controller::clear_world ()
 void
 Controller::undo ()
 {
+#ifdef DEBUG
   std::cout << "Controller::undo (): undostack: " << undo_world_stack.size() 
             << " redostack: " << redo_world_stack.size() << std::endl;
+#endif
 
   if (!undo_world_stack.empty())
     {
