@@ -1,4 +1,4 @@
-//  $Id: glut_display.cxx,v 1.10 2003/01/09 22:10:24 grumbel Exp $
+//  $Id: glut_display.cxx,v 1.23 2003/07/26 14:43:56 grumbel Exp $
 //
 //  Construo - A wire-frame construction gamee
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -17,6 +17,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#include <config.h>
+
 #ifdef HAVE_FREEGLUT
 #  include <GL/freeglut.h>
 #else
@@ -24,10 +26,12 @@
 #endif
 
 #include <stdio.h>
+#include <assert.h>
 #include <iostream>
 #include "buttons.hxx"
 #include "events.hxx"
 #include "settings.hxx"
+#include "construo_main.hxx"
 #include "screen_manager.hxx"
 #include "glut_display.hxx"
 
@@ -58,12 +62,17 @@ void keyboard_func (unsigned char key, int x, int y)
   GlutDisplay::instance()->keyboard_func(key, x, y);
 }
 
+void special_func (int key, int x, int y)
+{
+  GlutDisplay::instance()->special_func(key, x, y);
+}
+
 void mouse_motion_func (int x, int y)
 {
   GlutDisplay::instance()->mouse_motion_func(x, y);
 }
 
-GlutDisplay::GlutDisplay (int w, int h)
+GlutDisplay::GlutDisplay (int w, int h, int fullscreen)
 {
   instance_ = this;
 
@@ -78,7 +87,7 @@ GlutDisplay::GlutDisplay (int w, int h)
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
   glutInitWindowSize(width, height);
   //glutInitWindowPosition(100, 100); don't care
-  glutSetWindow(glutCreateWindow(argv[0]));
+  glutSetWindow(glutCreateWindow(construo_main->get_title()));
 
   glutDisplayFunc(::display_func);
   glutReshapeFunc(::reshape_func);
@@ -89,9 +98,8 @@ GlutDisplay::GlutDisplay (int w, int h)
 
   glutIdleFunc (::idle_func);
   glutKeyboardFunc(::keyboard_func);
+  glutSpecialFunc(::special_func);
 
-  is_fullscreen = false;
-  
   glClearColor (0.0, 0.0, 0.0, 0.1);
   if (settings.alphablending)
     {
@@ -105,8 +113,17 @@ GlutDisplay::GlutDisplay (int w, int h)
       glEnable(GL_LINE_SMOOTH);
     }
   
-  glEnable(GL_SCISSOR_TEST);
-  glScissor(0, 0, settings.screen_width, settings.screen_height);
+  //glEnable(GL_SCISSOR_TEST);
+  //glScissor(0, 0, settings.screen_width, settings.screen_height);
+  glutSetCursor(GLUT_CURSOR_FULL_CROSSHAIR);
+
+  window_x_pos = 0;
+  window_y_pos = 0;
+  window_width  = w;
+  window_height = h;
+
+  if (fullscreen)
+    enter_fullscreen();
 }
 
 void
@@ -121,6 +138,20 @@ GlutDisplay::~GlutDisplay()
 {
 }
 
+void
+GlutDisplay::draw_lines (std::vector<Line>& lines, Color color, int wide)
+{
+  glLineWidth (wide);
+  
+  glBegin (GL_LINES);
+  for (std::vector<Line>::const_iterator i = lines.begin(); i != lines.end(); ++i)
+    {
+      glVertex2f (i->x1, i->y1);
+      glVertex2f (i->x2, i->y2);     
+    }
+  glEnd ();
+}
+
 void 
 GlutDisplay::draw_line(float x1, float y1, float x2, float y2, Color color, int wide)
 {
@@ -159,10 +190,20 @@ GlutDisplay::draw_fill_rect(float x1, float y1, float x2, float y2, Color color)
   glEnd ();
 }
 
+void
+GlutDisplay::draw_circles(std::vector<Circle>& circles, Color color)
+{
+  for (std::vector<Circle>::iterator i = circles.begin(); i != circles.end(); ++i)
+    {
+      draw_circle(i->x, i->y, i->r, color);
+    }
+}
+
 void
 GlutDisplay::draw_circle(float x, float y, float r, Color color)
 {
   glColor4f (color.r, color.g, color.b, color.a);
+  
   GLUquadricObj* qobj = gluNewQuadric ();
   gluQuadricDrawStyle(qobj, GLU_SILHOUETTE);
   //gluQuadricNormals (qobj, GLU_FLAT);
@@ -263,9 +304,14 @@ GlutDisplay::flip (int x1, int y1, int x2, int y2)
 void
 GlutDisplay::reshape_func(int w, int h)
 {
+  std::cout << "Reshape: " << w << " " << h << std::endl;
   glViewport (0,0, w, h);
+  glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D (0, w, h, 0);
+  glMatrixMode(GL_MODELVIEW);
+  width  = w;
+  height = h;
 }
 
 void
@@ -318,18 +364,31 @@ void
 GlutDisplay::idle_func ()
 {
   /*  if (Controller::instance()->is_running() || update_display > 0)
-    {
+      {
       //system_context->sleep (0); // limit CPU usage via brute force
       update_display = 0;
-    }*/
+      }*/
   if (!ScreenManager::instance ()->is_finished())
     {
       ScreenManager::instance ()->run_once();
     }
   else
     {
-      // FIXME: is there a more gracefull way to end this?
-      exit(EXIT_SUCCESS);
+      construo_main->exit();
+    }
+}
+
+void
+GlutDisplay::special_func (int key, int x, int y)
+{
+  switch (key)
+    {
+    case GLUT_KEY_F11:
+      if (is_fullscreen)
+        leave_fullscreen();
+      else
+        enter_fullscreen();
+      break;
     }
 }
 
@@ -357,6 +416,9 @@ GlutDisplay::keyboard_func (unsigned char key, int x, int y)
     case 'q':
       event.button.id = BUTTON_ESCAPE;
       break;
+    case 'h':
+      event.button.id = BUTTON_FLIP;
+      break;
     case 'f':
       event.button.id = BUTTON_FIX;
       break;
@@ -364,6 +426,26 @@ GlutDisplay::keyboard_func (unsigned char key, int x, int y)
       event.button.id = BUTTON_DUPLICATE;
       break;
 
+    case 'v':
+      event.button.id = BUTTON_SETVELOCITY;
+      break;
+
+    case 'c':
+      event.button.id = BUTTON_CLEAR;
+      break;
+
+    case 'j':
+      event.button.id = BUTTON_JOIN;
+      break;
+
+    case 'a':
+      event.button.id = BUTTON_ACTIONCAM;
+      break;
+
+    case 'o':
+      event.button.id = BUTTON_HIDEDOTS;
+      break;
+
     case '!':
       event.button.id = BUTTON_QUICKLOAD1;
       break;
@@ -444,6 +526,10 @@ GlutDisplay::keyboard_func (unsigned char key, int x, int y)
       event.button.id = BUTTON_QUICKSAVE9;
       break;
 
+    case 'g':
+      event.button.id = BUTTON_GRID;
+      break;
+
     case 'u':
       event.button.id = BUTTON_UNDO;
       break;
@@ -458,8 +544,10 @@ GlutDisplay::keyboard_func (unsigned char key, int x, int y)
       event.button.id = BUTTON_ZOOM_OUT;
       break;
     default:
+#ifdef DEBUG
       std::cout << "GlutDisplay: Unhandled keypress: '" << key << "'[" << int(key) << "] x/y: " 
                 << x << ", " << y << std::endl;
+#endif
       return;
     }
 
@@ -475,21 +563,43 @@ GlutDisplay::mouse_motion_func (int x, int y)
 }
 
 void
-GlutDisplay::set_fullscreen (bool fullscreen)
+GlutDisplay::leave_fullscreen()
 {
-  if (fullscreen)
-    {        
-      char mode[64];
-      snprintf (mode, 64, "%dx%d:%d@%d", width, height, 16, 80);
-      std::cout << "GlutDisplay: switching to: " << mode << std::endl;
-      glutGameModeString(mode);
-      glutEnterGameMode();
-      is_fullscreen = true;
-    }
-  else
-    {
-      is_fullscreen = false;
-    }
+  std::cout << "GlutDisplay: leaving fullscreen: restoring to: pos: " 
+            << window_x_pos << ", " << window_y_pos << " - WxH: "
+            << window_width << ", " << window_height << std::endl;
+
+  glutReshapeWindow(window_width, window_height);
+  glutPositionWindow(window_x_pos, window_y_pos);
+
+  is_fullscreen = false;
+}
+
+void
+GlutDisplay::enter_fullscreen()
+{
+#if 0
+  char mode[64];
+  snprintf (mode, 64, "%dx%d:%d@%d", width, height, 16, 80);
+  std::cout << "GlutDisplay: switching to: " << mode << std::endl;
+  glutGameModeString(mode);
+  glutEnterGameMode();
+  is_fullscreen = true;
+#else
+  std::cout << "GlutDisplay: Entering fullscreen" << std::endl;
+
+  window_x_pos  = glutGet((GLenum)GLUT_WINDOW_X);
+  window_y_pos  = glutGet((GLenum)GLUT_WINDOW_Y);
+  window_width  = glutGet((GLenum)GLUT_WINDOW_WIDTH);
+  window_height = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
+  
+  std::cout << "Saving window: " << window_x_pos << ", " << window_y_pos << " - WxH: "
+            << window_width << ", " << window_height << std::endl;
+
+  glutFullScreen();
+
+  is_fullscreen = true;
+#endif
 }
 
 void
@@ -497,7 +607,56 @@ GlutDisplay::set_clip_rect (int x1, int y1, int x2, int y2)
 {
   //std::cout << "Setting cliprect: " << x1<< " " <<y1<< " " <<x2-x1+1<< " " <<y2-y1+1<<std::endl;
   // FIXME: doesn't really work for some reason
+  //std::cout << "Clip: " <<  x1 << ", " << y1 << " - " <<  x2-x1+1 << "x" << y2-y1+1 << std::endl;
   //glScissor(x1, y1, x2-x1+1, y2-y1+1);
 }
 
+void
+GlutDisplay::push_quick_draw()
+{
+  if (settings.antialiasing && settings.alphablending)
+    {
+      glDisable(GL_LINE_SMOOTH);
+    }
+}
+
+void
+GlutDisplay::pop_quick_draw()
+{
+  if (settings.antialiasing && settings.alphablending)
+    {
+      glEnable(GL_LINE_SMOOTH);
+    }
+}
+
+void
+GlutDisplay::set_cursor_real(CursorType cursor)
+{
+  switch(cursor)
+    {
+    case CURSOR_INSERT:
+      glutSetCursor(GLUT_CURSOR_CROSSHAIR);
+      break;
+    case CURSOR_SCROLL:
+      glutSetCursor(GLUT_CURSOR_INFO);
+      break;
+    case CURSOR_ZOOM:
+      glutSetCursor(GLUT_CURSOR_HELP);
+      break;
+    case CURSOR_COLLIDER:
+      glutSetCursor(GLUT_CURSOR_CROSSHAIR);
+      break;
+    case CURSOR_SELECT:
+      glutSetCursor(GLUT_CURSOR_INHERIT);
+      break;
+    case CURSOR_ROTATE:
+      glutSetCursor(GLUT_CURSOR_CYCLE);
+      break;
+    default:
+      std::cout << "GlutDisplay: Unhandled cursor type: " << cursor << std::endl;
+      break;
+    }
+
+}
+
 /* EOF */
