@@ -1,4 +1,4 @@
-//  $Id: gui_manager.cxx,v 1.31 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: gui_manager.cxx,v 1.36 2003/07/26 00:39:31 grumbel Exp $
 //
 //  Construo - A wire-frame construction game
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -31,6 +31,7 @@
 #include "worldview_component.hxx"
 #include "worldview_insert_tool.hxx"
 #include "globals.hxx"
+#include "root_graphic_context.hxx"
 #include "screen_manager.hxx"
 
 using namespace StringUtils;
@@ -108,6 +109,10 @@ GUIManager::process_button_events (ButtonEvent& button)
     {
       switch (button.id)
         {
+        case BUTTON_FULLSCREEN:
+          
+          break;
+
         case BUTTON_START:
           Controller::instance()->start_simulation ();
           break;
@@ -127,6 +132,15 @@ GUIManager::process_button_events (ButtonEvent& button)
         case BUTTON_FIX:
           current_component->on_fix_press (x, y);
           break;
+
+        case BUTTON_JOIN:
+          current_component->on_join_press (x, y);
+          break;
+
+        case BUTTON_GRID:
+          current_component->on_grid_press(x, y);
+          break;
+
         case BUTTON_DELETE:
           current_component->on_delete_press (x, y);
           break;
@@ -249,8 +263,10 @@ GUIManager::process_button_events (ButtonEvent& button)
           break;
 
         default:
+#ifdef DEBUG
           std::cout << "GUIManager:process_button_events: Got unhandled BUTTON_EVENT release: "
                     << button.id << std::endl;
+#endif
           break;
         }
     }
