@@ -1,4 +1,4 @@
-//  $Id: selection.cxx,v 1.6 2003/01/09 22:10:24 grumbel Exp $
+//  $Id: selection.cxx,v 1.10 2003/07/25 15:57:24 grumbel Exp $
 //
 //  Construo - A wire-frame construction gamee
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -17,6 +17,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+#include <config.h>
 #include <algorithm>
 #include <math.h>
 #include <map>
@@ -113,8 +114,11 @@ Selection::select_particles (Vector2d p1, Vector2d p2)
 {
   world = Controller::instance()->get_world ();
   
-  selection = world->get_particles (p1.x, p1.y,
-                                    p2.x, p2.y);
+  std::vector<Particle*> particles = world->get_particles (p1.x, p1.y,
+                                                           p2.x, p2.y);
+
+  selection = SelectionLst(particles.begin(),
+                           particles.end());
 }
 
 void
@@ -195,9 +199,52 @@ Selection::validate()
 {
   if (world != Controller::instance()->get_world ())
     {
-      std::cout << "World changed; " << world << " " << Controller::instance()->get_world () << std::endl;
+      //std::cout << "World changed; " << world << " " << Controller::instance()->get_world () << std::endl;
       clear();
     }
 }
 
+void
+Selection::join_doubles(float toleranz)
+{
+  // FIXME: Undo add undo, if stuff is going to change
+  Controller::instance()->push_undo();
+  World& world = *Controller::instance()->get_world ();
+  
+  for (SelectionLst::iterator i = selection.begin (); i != selection.end (); ++i)
+    {
+      SelectionLst::iterator j = i;
+      ++j;
+      for (; j != selection.end (); ++j)
+        {
+          if (Vector2d::distance((*j)->pos, (*i)->pos) < toleranz)
+            {
+              // Join two particles
+              std::cout << "joining particles: " << (*j)->pos << " " << (*i)->pos << std::endl;
+              (*j)->pos      = ((*j)->pos + (*i)->pos) * 0.5f;
+              (*j)->velocity = ((*j)->velocity + (*i)->velocity) * 0.5f;
+             
+              //selection.remove(*i);
+
+              { // Everything that is connected to the particle 'i'
+                // which should get removed, needs to get connected to
+                // 'j'
+                std::vector<Spring*>& springs = world.get_spring_mgr ();
+                for (std::vector<Spring*>::iterator s = springs.begin(); s != springs.end(); ++s)
+                  {
+                    if ((*s)->particles.first == (*i))
+                      (*s)->particles.first = (*j);
+                        
+                    if ((*s)->particles.second == (*i))
+                      (*s)->particles.second = (*j);
+                  }
+              }
+
+              world.remove_particle(*i);
+            }
+        }
+    }
+  clear();
+}
+
 /* EOF */
