@@ -1,4 +1,4 @@
-//  $Id: unix_system.cxx,v 1.10 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: unix_system.cxx,v 1.15 2003/07/25 13:16:04 grumbel Exp $
 //
 //  Construo - A wire-frame construction game
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -18,6 +18,7 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <stdio.h>
+#include <assert.h>
 #include <unistd.h>
 #include <sys/time.h>
 #include <time.h>
@@ -32,6 +33,7 @@
 #include "construo_error.hxx"
 #include "string_utils.hxx"
 #include "construo.hxx"
+#include "path_manager.hxx"
 #include "unix_system.hxx"
 
 using namespace StringUtils;
@@ -173,7 +175,7 @@ UnixSystem::get_file_type(const std::string& filename)
         }
       else if (S_ISREG(buf.st_mode))
         {
-          if (is_suffix(filename, ".construo"))
+          if (has_suffix(filename, ".construo") || has_suffix(filename, ".construo.gz"))
             return FT_CONSTRUO_FILE;
           else
             {
@@ -195,13 +197,13 @@ UnixSystem::translate_filename (const std::string& filename)
       assert("root directory is not translatable");
       return "";
     }
-  else if (is_prefix(filename, "/user/"))
+  else if (has_prefix(filename, "/user/"))
     {
-      return "/home/ingo/.construo/" + filename.substr(6); 
+      return construo_rc_path + filename.substr(6);
     }
-  else if (is_prefix(filename, "/examples/"))
-    {
-      return "examples/" + filename.substr(10); 
+  else if (has_prefix(filename, "/examples/"))
+    { 
+      return path_manager.complete("examples/") + filename.substr(10); 
     }
   else
     return filename;
@@ -222,6 +224,8 @@ UnixSystem::open_output_file(const std::string& filename)
 }
 
 
+/** Sort directories before files and sort them all
+    alphabetically */
 struct DirectorySorter
 {
   std::string pathname;
@@ -269,20 +273,28 @@ UnixSystem::read_directory(const std::string& arg_pathname)
       std::string pathname = translate_filename (arg_pathname);
 
       DIR* dir = ::opendir (pathname.c_str());
-      // FIXME: Error checking here
-      struct dirent* entry;
-
-      while ((entry = readdir(dir)) != 0)
+      
+      if (!dir)
         {
-          if (strcmp(entry->d_name, ".") != 0
-              && strcmp(entry->d_name, "..") != 0
-              && strcmp(entry->d_name, "CVS") != 0)
-            { // We ignore unusefull directories
-              dir_lst.push_back(entry->d_name);
-            }
+          std::cout << "UnixSystem: Error couldn't open: '" << pathname << "', ignoring\n"
+                    << "            error and continuing with an empty directory" << std::endl;
         }
+      else
+        {
+          struct dirent* entry;
+
+          while ((entry = readdir(dir)) != 0)
+            {
+              if (strcmp(entry->d_name, ".") != 0
+                  && strcmp(entry->d_name, "..") != 0
+                  && strcmp(entry->d_name, "CVS") != 0)
+                { // We ignore unusefull directories
+                  dir_lst.push_back(entry->d_name);
+                }
+            }
   
-      closedir (dir);
+          closedir (dir);
+        }
 
       std::sort(dir_lst.begin(), dir_lst.end(), DirectorySorter(pathname));
 
