@@ -1,4 +1,4 @@
-//  $Id: world_gui_manager.cxx,v 1.6 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: world_gui_manager.cxx,v 1.12 2003/07/26 16:34:57 grumbel Exp $
 //
 //  Construo - A wire-frame construction gamee
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -29,6 +29,7 @@
 #include "gui_label.hxx"
 #include "globals.hxx"
 #include "screen_manager.hxx"
+#include "root_graphic_context.hxx"
 #include "world_gui_manager.hxx"
 
 using namespace StringUtils;
@@ -112,6 +113,16 @@ bool action_cam_hfunc ()
   return Controller::instance()->get_action_cam();
 }
 
+bool show_grid_hfunc() 
+{
+  return WorldViewComponent::instance()->uses_grid();
+}
+
+void show_grid_callback()
+{
+  WorldViewComponent::instance()->on_grid_press(0, 0);
+}
+
 void redo_callback ()
 {
   return Controller::instance()->redo();
@@ -122,7 +133,7 @@ void undo_callback ()
   return Controller::instance()->undo();
 }
 
-#define BUTTON_POS(n) (75 + n * 30)
+#define BUTTON_POS(n) (80 + n * 30)
 #define BUTTON_RPOS(n) (50 + n * 30)
 #define BUTTON_WIDTH  75
 #define BUTTON_HEIGHT 25
@@ -139,13 +150,14 @@ WorldGUIManager::WorldGUIManager ()
   //add(new GUIZoomInButton ());
   //add(new GUIZoomOutButton ());
   add(new GUILoadButton ());
-  add(new GUIGenericButton ("Save", 10, BUTTON_POS(9), BUTTON_WIDTH, BUTTON_HEIGHT, save_button_callback));
+  add(new GUIGenericButton ("Save", 10, BUTTON_POS(10), BUTTON_WIDTH, BUTTON_HEIGHT, save_button_callback));
 
-  add(new GUIGenericButton ("Undo", 10, BUTTON_POS(5), BUTTON_WIDTH, BUTTON_HEIGHT, undo_callback));
-  add(new GUIGenericButton ("Redo", 10, BUTTON_POS(6), BUTTON_WIDTH, BUTTON_HEIGHT, redo_callback));
+  add(new GUIGenericButton ("Undo", 10, BUTTON_POS(6), BUTTON_WIDTH, BUTTON_HEIGHT, undo_callback));
+  add(new GUIGenericButton ("Redo", 10, BUTTON_POS(7), BUTTON_WIDTH, BUTTON_HEIGHT, redo_callback));
 
   add(new GUIGenericButton ("ActionCam", 10, BUTTON_POS(2), BUTTON_WIDTH, BUTTON_HEIGHT, action_cam_callback, action_cam_hfunc));
   add(new GUIGenericButton ("Hide Dots", 10, BUTTON_POS(3), BUTTON_WIDTH, BUTTON_HEIGHT, hide_dots_callback, hide_dots_hfunc));
+  add(new GUIGenericButton ("Use Grid", 10, BUTTON_POS(4), BUTTON_WIDTH, BUTTON_HEIGHT, show_grid_callback, show_grid_hfunc));
   add(new GUIQuitButton ());
 
   //add(new GUILabel ("Tools", BUTTON_LX_POS, BUTTON_POS(3)+5, BUTTON_WIDTH, BUTTON_HEIGHT));
@@ -158,18 +170,19 @@ WorldGUIManager::WorldGUIManager ()
   add(new GUIGenericButton ("-", BUTTON_LX_POS + 38, BUTTON_RPOS(8), 25, 25, zoom_out_callback));
   add(new GUIGenericButton ("+", BUTTON_LX_POS +  6, BUTTON_RPOS(8), 25, 25, zoom_in_callback));
 
+  // FIXME: Stuff for particle mass and Co. must be implemented in another way
   if(0)
     {
-  add(new GUIGenericButton ("Increase ParticleMass",   650, 220, 140, 25, increase_particle_mass));
-  add(new GUIGenericButton ("Decrease ParticleMass",   650, 250, 140, 25, decrease_particle_mass));
+      add(new GUIGenericButton ("Increase ParticleMass",   650, 220, 140, 25, increase_particle_mass));
+      add(new GUIGenericButton ("Decrease ParticleMass",   650, 250, 140, 25, decrease_particle_mass));
 
-  add(new GUILabel ("Stiffness",   550, 280, 75, 25));
+      add(new GUILabel ("Stiffness",   550, 280, 75, 25));
 
-  add(new GUIGenericButton ("+",   BUTTON_LX_POS, 280, 25, 25, increase_particle_mass));
-  add(new GUIGenericButton ("-",   680, 280, 25, 25, decrease_particle_mass));
+      add(new GUIGenericButton ("+",   BUTTON_LX_POS, 280, 25, 25, increase_particle_mass));
+      add(new GUIGenericButton ("-",   680, 280, 25, 25, decrease_particle_mass));
 
-  add(new GUIGenericButton ("+",   650, 280, 25, 25, increase_particle_mass));
-  add(new GUIGenericButton ("-",   680, 280, 25, 25, decrease_particle_mass));
+      add(new GUIGenericButton ("+",   650, 280, 25, 25, increase_particle_mass));
+      add(new GUIGenericButton ("-",   680, 280, 25, 25, decrease_particle_mass));
     }
   //add(new GUIWindow ("Test Window",   200, 100, 200, 90));
  
@@ -193,16 +206,17 @@ WorldGUIManager::update()
 void
 WorldGUIManager::draw_overlay ()
 {
-  graphic_context->draw_string (10, 20, "      [1-9] - quick save");
-  graphic_context->draw_string (10, 32, "[shift 1-9] - quick load");
-  graphic_context->draw_string (10, 44, "    [space] - run simulation");
-  graphic_context->draw_string (10, 56, "      [tab] - toggle slow motion");
+  graphic_context->draw_string (10,  20, "      [1-9] - quick save");
+  graphic_context->draw_string (10,  32, "[shift 1-9] - quick load");
+  graphic_context->draw_string (10,  44, "    [space] - run simulation");
+  graphic_context->draw_string (10,  56, "      [tab] - toggle slow motion");
+  graphic_context->draw_string (10,  68, "      [F11] - toggle fullscreen");
 
   graphic_context->draw_string (200,  20, "     [c] - clear scene");
   graphic_context->draw_string (200,  32, "     [u] - undo to last state");
   graphic_context->draw_string (200,  44, "     [r] - redo (undo an undo)");
   graphic_context->draw_string (200,  56, "   [+/-] - zoom in/out");
-
+  graphic_context->draw_string (200,  68, "     [g] - toggle grid");
 
   graphic_context->draw_string (600,  32, "[middle] - scroll");
 
@@ -217,14 +231,17 @@ WorldGUIManager::draw_overlay ()
     case WorldViewComponent::SELECT_MODE:
       graphic_context->draw_string (600,  20, "  [left] - create/move selection");
       graphic_context->draw_string (600,  44, " [right] - rotate selection");
+
       graphic_context->draw_string (400,  20, "     [v] - set velocity");
       graphic_context->draw_string (400,  32, "     [d] - duplicate selection");
       graphic_context->draw_string (400,  44, "     [h] - flip selection");
       graphic_context->draw_string (400,  56, "     [f] - fix selection");
+      graphic_context->draw_string (400,  68, "     [j] - join dots");
       break;
       
     case WorldViewComponent::ZOOM_MODE:
       graphic_context->draw_string (600,  20, "  [left] - zoom into region");
+      graphic_context->draw_string (600,  44, " [right] - zoom out");
       break;
 
     case WorldViewComponent::COLLIDER_MODE:
@@ -239,8 +256,8 @@ WorldGUIManager::draw_overlay ()
   World& world = *Controller::instance()->get_world ();
 
   /*graphic_context->draw_string (600,  430, "Particles Mass: ");
-  graphic_context->draw_string (BUTTON_LX_POS,  430, 
-                                to_string(WorldViewComponent::instance()->get_insert_tool()->get_particle_mass ()));
+    graphic_context->draw_string (BUTTON_LX_POS,  430, 
+    to_string(WorldViewComponent::instance()->get_insert_tool()->get_particle_mass ()));
   */
   int bottom_line = graphic_context->get_height() - 10;
   graphic_context->draw_string (10, bottom_line-20, "FPS: ");
