@@ -1,4 +1,4 @@
-//  $Id: worldview_select_tool.cxx,v 1.17 2003/01/11 19:07:48 grumbel Exp $
+//  $Id: worldview_select_tool.cxx,v 1.21 2003/07/26 16:15:22 grumbel Exp $
 //
 //  Construo - A wire-frame construction gamee
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -26,6 +26,7 @@
 #include "particle.hxx"
 #include "world_gui_manager.hxx"
 #include "rect.hxx"
+#include "root_graphic_context.hxx"
 #include "worldview_select_tool.hxx"
 
 WorldViewSelectTool::WorldViewSelectTool ()
@@ -127,6 +128,9 @@ WorldViewSelectTool::on_primary_button_press (int screen_x, int screen_y)
   WorldGUIManager::instance()->grab_mouse (WorldViewComponent::instance());
 
   mode = GETTING_SELECTION_MODE;
+  
+  click_pos.x = x;
+  click_pos.y = y;
 
   // If the mouse clicks on a particle from the selection, we move the selection
   Particle* new_current_particle = world.get_particle (x, y);
@@ -136,6 +140,7 @@ WorldViewSelectTool::on_primary_button_press (int screen_x, int screen_y)
         {
           Controller::instance()->push_undo();
           mode = MOVING_SELECTION_MODE;
+          move_diff = Vector2d();
           break;
         }
     }
@@ -144,8 +149,6 @@ WorldViewSelectTool::on_primary_button_press (int screen_x, int screen_y)
   if (mode == GETTING_SELECTION_MODE)
     {
       selection.clear ();
-      click_pos.x = x;
-      click_pos.y = y;
     }
 }
 
@@ -170,6 +173,8 @@ void
 WorldViewSelectTool::on_secondary_button_press (int screen_x, int screen_y)
 {
   Controller::instance()->push_undo();
+  graphic_context->push_cursor();
+  graphic_context->set_cursor(CURSOR_ROTATE);
 
   mode = ROTATING_SELECTION_MODE;
   WorldGUIManager::instance()->grab_mouse (WorldViewComponent::instance());  
@@ -188,6 +193,7 @@ WorldViewSelectTool::on_secondary_button_press (int screen_x, int screen_y)
 void
 WorldViewSelectTool::on_secondary_button_release (int x, int y)
 {
+  graphic_context->pop_cursor();
   WorldGUIManager::instance()->ungrab_mouse (WorldViewComponent::instance());
   mode = IDLE_MODE;
 }
@@ -242,23 +248,44 @@ WorldViewSelectTool::on_mouse_move (int screen_x, int screen_y, int of_x, int of
   switch (mode)
     {
     case MOVING_SELECTION_MODE:
-      for (Selection::iterator i = selection.begin (); i != selection.end (); ++i)
-        {
-          // Will lead to round errors 
-          (*i)->pos.x += of_x / WorldViewComponent::instance()->get_gc ()->get_zoom();
-          (*i)->pos.y += of_y / WorldViewComponent::instance()->get_gc ()->get_zoom();
+      {
+        Vector2d new_pos(WorldViewComponent::instance()->get_gc()->screen_to_world_x (screen_x),
+                         WorldViewComponent::instance()->get_gc()->screen_to_world_y (screen_y));      
 
-          std::vector<Spring*>& spring_mgr = world.get_spring_mgr();
-          for (std::vector<Spring*>::iterator j = spring_mgr.begin (); 
-               j != spring_mgr.end (); ++j)
-            {
-              if ((*j)->particles.first == *i
-                  || (*j)->particles.second == *i)
-                {
-                  (*j)->recalc_length ();
-                }
-            }
-        }
+        Vector2d diff = new_pos - click_pos;
+
+        // Undo the last move (FIXME: Potential round errors)
+        for (Selection::iterator i = selection.begin (); i != selection.end (); ++i)
+          {
+            (*i)->pos -= move_diff;
+          }
+
+        if (WorldViewComponent::instance()->uses_grid())
+          {
+            diff.x = Math::round_to(diff.x, 10);
+            diff.y = Math::round_to(diff.y, 10);
+          }
+
+        move_diff = diff;
+
+        for (Selection::iterator i = selection.begin (); i != selection.end (); ++i)
+          {
+            (*i)->pos += diff;
+
+            // Recalculate all springs that are attached to the
+            // selection, but not fully in it.
+            std::vector<Spring*>& spring_mgr = world.get_spring_mgr();
+            for (std::vector<Spring*>::iterator j = spring_mgr.begin (); 
+                 j != spring_mgr.end (); ++j)
+              {
+                if ((*j)->particles.first == *i
+                    || (*j)->particles.second == *i)
+                  {
+                    (*j)->recalc_length ();
+                  }
+              }
+          }
+      }
       break;
     case ROTATING_SELECTION_MODE:
       {
@@ -306,4 +333,11 @@ WorldViewSelectTool::on_button_press (int button_id, int x, int y)
     }
 }
 
+void
+WorldViewSelectTool::on_join_press (int x, int y)
+{
+  std::cout << "Join pressed" << std::endl;
+  selection.join_doubles(5.0f);
+}
+
 /* EOF */
