@@ -1,4 +1,4 @@
-//  $Id: x11_display.cxx,v 1.31 2003/01/11 16:11:36 grumbel Exp $
+//  $Id: x11_display.cxx,v 1.41 2003/07/26 14:55:48 grumbel Exp $
 //
 //  Construo - A wire-frame construction game
 //  Copyright (C) 2002 Ingo Ruhnke <grumbel@gmx.de>
@@ -19,8 +19,24 @@
 
 #include <iostream>
 #include <X11/Xutil.h>
+#include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
+#include "cursor_insert.xbm"
+#include "cursor_insert_mask.xbm"
+
+#include "cursor_select.xbm"
+#include "cursor_select_mask.xbm"
+
+#include "cursor_zoom.xbm"
+#include "cursor_zoom_mask.xbm"
+
+#include "cursor_scroll.xbm"
+#include "cursor_scroll_mask.xbm"
+
+#include "cursor_collider.xbm"
+#include "cursor_collider_mask.xbm"
+
 #include "construo_error.hxx"
 #include "config.h"
 #include "x11_display.hxx"
@@ -34,14 +50,22 @@ extern ConstruoMain* construo_main;
 Atom wm_delete_window;
 
 static char zoom_tool_cursor[] = {
-    /* -------- -------- */  0x00,
-    /* -------- -------- */  0x00,
-    /* ------xx xx------ */  0x18,                         
-    /* ----xxxx xxxx---- */  0x3c,
-    /* ----xxxx xxxx---- */  0x3c,
-    /* ------xx xx------ */  0x18,
-    /* -------- -------- */  0x00,
-    /* -------- -------- */  0x00
+  /* -------- -------- */  0x00,
+  /* -------- -------- */  0x00,
+  /* ------xx xx------ */  0x18,                         
+  /* ----xxxx xxxx---- */  0x3c,
+  /* ----xxxx xxxx---- */  0x3c,
+  /* ------xx xx------ */  0x18,
+  /* -------- -------- */  0x00,
+  /* -------- -------- */  0x00,
+  /* -------- -------- */  0x00,
+  /* -------- -------- */  0x00,
+  /* ------xx xx------ */  0x18,                         
+  /* ----xxxx xxxx---- */  0x3c,
+  /* ----xxxx xxxx---- */  0x3c,
+  /* ------xx xx------ */  0x18,
+  /* -------- -------- */  0x00,
+  /* -------- -------- */  0x00
 };
 
 X11Display::X11Display(int w, int h, bool fullscreen_)
@@ -59,6 +83,7 @@ X11Display::X11Display(int w, int h, bool fullscreen_)
 
   attributes.background_pixel  = BlackPixel(display, screen);
   attributes.border_pixel      = WhitePixel(display, screen);
+
   if (fullscreen)
     attributes.override_redirect = True;
   else
@@ -74,15 +99,17 @@ X11Display::X11Display(int w, int h, bool fullscreen_)
     StructureNotifyMask  |
     ExposureMask;
 
+  colormap = DefaultColormap (display, screen);
+  attributes.colormap = colormap;
   window = XCreateWindow(display, RootWindow(display, screen),
                          0,0, // position
                          width, height, 0,
                          CopyFromParent, InputOutput, CopyFromParent, 
-                         CWOverrideRedirect | CWBackPixel | CWBorderPixel | CWEventMask,
+                         CWOverrideRedirect | CWBackPixel | CWBorderPixel | CWEventMask | CWColormap,
                          &attributes);
 
   { // Communicate a bit with the window manager
-    char *title = "Construo " VERSION;
+    char *title = construo_main->get_title();
 
     XTextProperty text_property;
     XStringListToTextProperty(&title, 1, &text_property);
@@ -126,32 +153,72 @@ X11Display::X11Display(int w, int h, bool fullscreen_)
   XGCValues gcv;
   gcv.foreground = 0xFFFFFF;
   gcv.background = 0x000000;
-  gcv.line_width = 2;
+
+  if (settings.thick_lines)
+    gcv.line_width = 2;
+  else
+    gcv.line_width = 0;
+
   gc = XCreateGC(display, window, 
                  GCLineWidth | GCForeground | GCBackground,
                  &gcv);
 
   if (fullscreen)
-    set_fullscreen (true);
+    set_fullscreen();
 
-  if (0) // custom cursor code
-    {
-      XColor cursor_fg;
-      XColor cursor_bg;
+  {
+    // Black&White
+    XColor cursor_fg = get_xcolor(Color(1.0f, 1.0f, 1.0f));
+    XColor cursor_bg = get_xcolor(Color(0, 0, 0));
+      
+    cursor_scroll_pix = XCreateBitmapFromData (display, window, (char*)cursor_scroll_bits, 
+                                           cursor_scroll_width, cursor_scroll_height);
+    cursor_scroll_mask = XCreateBitmapFromData (display, window, (char*)cursor_scroll_mask_bits, 
+                                                cursor_scroll_width, cursor_scroll_height);
+    cursor_scroll = XCreatePixmapCursor(display, cursor_scroll_pix, cursor_scroll_mask, &cursor_bg, &cursor_fg, 
+                                        cursor_scroll_x_hot, cursor_scroll_y_hot);
 
-      //set colors here
+    cursor_zoom_pix =  XCreateBitmapFromData (display, window, (char*)cursor_zoom_bits, 
+                                              cursor_zoom_width, cursor_zoom_height);
+    cursor_zoom_mask = XCreateBitmapFromData (display, window, (char*)cursor_zoom_mask_bits, 
+                                              cursor_zoom_width, cursor_zoom_height);
+    cursor_zoom = XCreatePixmapCursor(display, cursor_zoom_pix, cursor_zoom_mask, &cursor_bg, &cursor_fg, 
+                                      cursor_zoom_x_hot, cursor_zoom_y_hot);
 
-      Pixmap cursor_pm = XCreateBitmapFromData (display, window, zoom_tool_cursor, 8, 8);
-      Cursor cursor = XCreatePixmapCursor(display, cursor_pm, None, &cursor_bg, &cursor_fg, 4, 4);
-      XDefineCursor (display, window, cursor);
-    }
+    cursor_insert_pix =  XCreateBitmapFromData (display, window, (char*)cursor_insert_bits, 
+                                                cursor_insert_width, cursor_insert_height);
+    cursor_insert_mask = XCreateBitmapFromData (display, window, (char*)cursor_insert_mask_bits, 
+                                                cursor_insert_width, cursor_insert_height);
+    cursor_insert = XCreatePixmapCursor(display, cursor_insert_pix, cursor_insert_mask, &cursor_bg, &cursor_fg, 
+                                        cursor_insert_x_hot, cursor_insert_y_hot);
+
+    cursor_select_pix =  XCreateBitmapFromData (display, window, (char*)cursor_select_bits, 
+                                                cursor_select_width, cursor_select_height);
+    cursor_select_mask = XCreateBitmapFromData (display, window, (char*)cursor_select_mask_bits, 
+                                                cursor_select_width, cursor_select_height);
+    cursor_select = XCreatePixmapCursor(display, cursor_select_pix, cursor_select_mask, &cursor_bg, &cursor_fg, 
+                                        cursor_select_x_hot, cursor_select_y_hot);
+
+    cursor_collider_pix =  XCreateBitmapFromData (display, window, (char*)cursor_collider_bits, 
+                                                  cursor_collider_width, cursor_collider_height);
+    cursor_collider_mask = XCreateBitmapFromData (display, window, (char*)cursor_collider_mask_bits, 
+                                                  cursor_collider_width, cursor_collider_height);
+    cursor_collider = XCreatePixmapCursor(display, cursor_collider_pix, cursor_collider_mask, &cursor_bg, &cursor_fg, 
+                                          cursor_collider_x_hot, cursor_collider_y_hot);
+  }
+
+  set_cursor(CURSOR_INSERT);
 }
 
 X11Display::~X11Display ()
 {
   std::cout << "Closing X11 display" << std::endl;
+  
   if (fullscreen)
-    restore_mode ();
+    {
+      std::cout << "X11Display: Restoring video mode" << std::endl;
+      restore_mode ();
+    }
   
   if (doublebuffer)
     XFreePixmap (display, drawable);
@@ -160,17 +227,78 @@ X11Display::~X11Display ()
   XCloseDisplay(display); 
 }
 
+void
+X11Display::set_cursor_real(CursorType cursor)
+{
+  switch(cursor)
+    {
+    case CURSOR_INSERT:
+      XDefineCursor (display, window, cursor_insert);
+      break;
+    case CURSOR_SCROLL:
+      XDefineCursor (display, window, cursor_scroll);
+      break;
+    case CURSOR_ZOOM:
+      XDefineCursor (display, window, cursor_zoom);
+      break;
+    case CURSOR_COLLIDER:
+      XDefineCursor (display, window, cursor_collider);
+      break;
+    case CURSOR_SELECT:
+      XDefineCursor (display, window, cursor_select);
+      break;
+    default:
+      std::cout << "X11Display: Unhandled cursor type: " << cursor << std::endl;
+      break;
+    }
+}
+
+void
+X11Display::draw_lines (std::vector<Line>& lines, Color color, int wide)
+{
+  std::vector<XSegment> segments (lines.size());
+  
+  for (std::vector<Line>::size_type i = 0; i < lines.size(); ++i)
+    {
+      segments[i].x1 = static_cast<short>(lines[i].x1);
+      segments[i].y1 = static_cast<short>(lines[i].y1);
+      segments[i].x2 = static_cast<short>(lines[i].x2);      
+      segments[i].y2 = static_cast<short>(lines[i].y2);
+    }
+
+  XDrawSegments(display, drawable, gc, &*segments.begin(), segments.size());
+}
+
+void
+X11Display::draw_circles(std::vector<Circle>& circles, Color color)
+{
+  std::vector<XArc> arcs (circles.size());
+  for (std::vector<Circle>::size_type i = 0; i < circles.size(); ++i)
+    {
+      arcs[i].x = static_cast<short>(circles[i].x - circles[i].r);
+      arcs[i].y = static_cast<short>(circles[i].y - circles[i].r);
+      arcs[i].width  = static_cast<short>(2 * circles[i].r);
+      arcs[i].height = static_cast<short>(2 * circles[i].r);
+      arcs[i].angle1 = 0;
+      arcs[i].angle2 = 360 * 64;
+    }
+
+  XSetForeground(display, gc, get_color_value(color));
+  XFillArcs(display, drawable, gc, 
+            &*arcs.begin(), arcs.size());
+}
+
 void
 X11Display::draw_line(float x1, float y1, float x2, float y2, Color color, int wide)
 {
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XDrawLine (display, drawable, gc, (int) x1, (int) y1, (int) x2, (int) y2);
 }
 
 void
 X11Display::draw_fill_rect(float x1, float y1, float x2, float y2, Color color)
 {
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XFillRectangle (display, drawable, gc, 
                   int(x1), int(y1), 
                   int(x2 - x1), int(y2 - y1));
@@ -180,7 +308,7 @@ void
 X11Display::draw_fill_circle(float x, float y, float r, Color color)
 {
   // FIXME: doesn't work
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XFillArc(display, drawable, gc, 
            int(x-r), int(y-r),
            int(r*2), int(r*2), 0, 
@@ -191,14 +319,14 @@ void
 X11Display::draw_circle(float x, float y, float r, Color color)
 {
   // FIXME: doesn't work
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XDrawArc(display, drawable, gc, int(x-r), int(y-r), int(r*2.0f), int(r*2.0f), 0, 360*64);
 }
 
 void
 X11Display::draw_rect(float x1, float y1, float x2, float y2, Color color)
 {
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XDrawRectangle (display, drawable, gc, 
                   int(x1), int(y1), 
                   int(x2 - x1), int(y2 - y1));
@@ -207,14 +335,14 @@ X11Display::draw_rect(float x1, float y1, float x2, float y2, Color color)
 void
 X11Display::draw_string(float x, float y, const std::string& str, Color color)
 {
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XDrawString (display, drawable, gc, int(x), int(y), str.c_str (), str.length ());
 }
 
 void
 X11Display::draw_string_centered(float x, float y, const std::string& str, Color color)
 {
-  XSetForeground(display, gc, color.get_rgb());
+  XSetForeground(display, gc, get_color_value(color));
   XDrawString (display, drawable, gc, 
                int(x) - ((str.length() * 6) / 2), int(y), 
                str.c_str (), str.length ());
@@ -318,6 +446,12 @@ X11Display::read_event ()
             
         switch (sym)
           {
+          case XK_F11:
+            // FIXME: Shortcut
+            //send_button_press(BUTTON_FULLSCREEN);
+            toggle_fullscreen();
+            break;
+
           case XK_Left:
             send_button_press(BUTTON_SCROLL_LEFT);
             break;
@@ -338,6 +472,10 @@ X11Display::read_event ()
             send_button_press(BUTTON_ACTIONCAM);
             break;
 
+          case XK_j:
+            send_button_press(BUTTON_JOIN);
+            break;
+
           case XK_o:
             send_button_press(BUTTON_HIDEDOTS);
             break;
@@ -380,6 +518,9 @@ X11Display::read_event ()
           case XK_space:
             send_button_press(BUTTON_RUN);
             break;
+          case XK_g:            
+            send_button_press(BUTTON_GRID);
+            break;
           case XK_Tab:
             send_button_press(BUTTON_TOGGLESLOWMO);
             break;
@@ -424,7 +565,7 @@ X11Display::read_event ()
             break;
               
           default:
-            std::cout << "X11Display: unhandled keypress: " << sym << " " << XK_f << std::endl;
+            std::cout << "X11Display: unhandled keypress: " << sym << " " << XK_grave << std::endl;
             break;
           }
       }
@@ -559,11 +700,35 @@ X11Display::flip ()
 }
 
 void
-X11Display::set_fullscreen (bool fullscreen)
+X11Display::save_mode()
+{
+#ifdef HAVE_LIBXXF86VM
+  memset(&orig_modeline, 0, sizeof(orig_modeline));
+
+  // Get the current display settings for later restore
+  XF86VidModeGetModeLine(display, 
+                         DefaultScreen(display),
+                         &orig_dotclock,
+                         &orig_modeline);
+
+  XF86VidModeGetViewPort(display,
+                         DefaultScreen(display),
+                         &orig_viewport_x,
+                         &orig_viewport_y);
+  std::cout << "save_mode: "
+            << orig_dotclock << " " 
+            << orig_viewport_x << ", " << orig_viewport_y << std::endl;
+
+#endif /* HAVE_LIBXXF86VM */
+}
+
+void
+X11Display::set_fullscreen ()
 {
 #ifdef HAVE_LIBXXF86VM
   int event_base;
   int error_base;
+
   if (XF86VidModeQueryExtension(display, &event_base, &error_base) != True)
     {
       // No VidMode extension available, bailout
@@ -571,26 +736,14 @@ X11Display::set_fullscreen (bool fullscreen)
       return;
     }
 
-  int screen = DefaultScreen(display);
+  save_mode();
 
-  memset(&orig_modeline, 0, sizeof(orig_modeline));
-  // Get the current display settings for later restore
-  XF86VidModeGetModeLine(display, 
-                         screen,
-                         &orig_dotclock,
-                         &orig_modeline);
-
-  XF86VidModeGetViewPort(display,
-                         screen,
-                         &orig_viewport_x,
-                         &orig_viewport_y);
-
-  XF86VidModeModeInfo **modes;
+  XF86VidModeModeInfo **modes; 
   int nmodes;
   int mode_index = -1;
   if (XF86VidModeGetAllModeLines(display,
-                                 screen,
-                                 &nmodes,&modes))
+                                 DefaultScreen(display),
+                                 &nmodes,&modes)) // FIXME: memleak
     {
       std::cout << "VideoModes: (searching for " << width << "x" << height << ")" << std::endl;
       for (int i = 0; i < nmodes; i++)
@@ -623,20 +776,37 @@ X11Display::set_fullscreen (bool fullscreen)
                     << std::endl;
 
           if(XF86VidModeSwitchToMode(display,
-                                     screen,
+                                     DefaultScreen(display),
                                      modes[mode_index]))
             {
-              XF86VidModeSetViewPort(display, screen, 0, 0);
+              fullscreen = true;
+
+              { // Now that we have switched to the correct mode, we
+                // need to position the Viewport correct to the window
+                Window child_window;
+                int x, y;
+                // Get the windows absolute position (aka relative to
+                // the root window)
+                XTranslateCoordinates(display, window, DefaultRootWindow(display), 
+                                      0, 0, 
+                                      &x, &y, &child_window);
+                XF86VidModeSetViewPort(display, DefaultScreen(display), x, y);
+              }
+              
               // Hijack the focus (works only till the next focus change)
               XSetInputFocus(display, window, RevertToParent, CurrentTime);
-              
+
               // Capture the pointer
-              if (XGrabPointer(display, window, true, 0, GrabModeAsync, GrabModeAsync, window, None, CurrentTime)
-                  != GrabSuccess)
+              if (XGrabPointer(display, window, true, 0, GrabModeAsync, GrabModeAsync, 
+                               window, None, CurrentTime) != GrabSuccess)
                 {
                   std::cout << "X11Display: Couldn't grab the pointer" << std::endl;
                 }
             }
+          else
+            {
+              std::cout << "X11Display: Throuble switiching to fullscreen?!" << std::endl;
+            }
         }
       else // No mode found
         {
@@ -663,6 +833,8 @@ X11Display::run()
 {
   while (!ScreenManager::instance ()->is_finished ())
     {
+      ScreenManager::instance ()->run_once();
+
       if (Controller::instance()->is_running())
         {
           system_context->sleep (0); // limit CPU usage via brute force
@@ -672,14 +844,26 @@ X11Display::run()
         {
           wait_for_events_blocking();
         }
-      ScreenManager::instance ()->run_once();
     }
 }
 
+void
+X11Display::toggle_fullscreen()
+{
+  std::cout << "Fullscreen state: " << fullscreen << std::endl;
+
+  if (fullscreen)
+    restore_mode();
+  else
+    set_fullscreen();
+}
+
 void
 X11Display::restore_mode ()
 {
 #ifdef HAVE_LIBXXF86VM
+  std::cout << "X11Display::restore_mode()" << std::endl;
+
   XF86VidModeModeInfo modeinfo;
   
   modeinfo.dotclock   = orig_dotclock;
@@ -703,6 +887,8 @@ X11Display::restore_mode ()
   XF86VidModeSetViewPort(display, DefaultScreen(display),
                          orig_viewport_x, orig_viewport_y);
 
+  XUngrabPointer(display, CurrentTime);
+
   fullscreen = false;
 #endif
 }
@@ -723,4 +909,33 @@ X11Display::set_clip_rect (int x1, int y1, int x2, int y2)
                       Unsorted);
 }
 
+unsigned int
+X11Display::get_color_value(const Color& color)
+{
+  XColor x_color;
+
+  x_color.red   = int(color.r * 65535);
+  x_color.green = int(color.g * 65535);
+  x_color.blue  = int(color.b * 65535);
+
+  XAllocColor(display, colormap, &x_color);
+
+  return x_color.pixel;
+}
+
+XColor
+X11Display::get_xcolor(const Color& color)
+{
+  XColor x_color;
+
+  x_color.red   = int(color.r * 65535);
+  x_color.green = int(color.g * 65535);
+  x_color.blue  = int(color.b * 65535);
+
+  XAllocColor(display, colormap, &x_color);
+
+  return x_color;
+ 
+}
+
 /* EOF */
